import type { RobotGroup } from '@api/robot';
import http from '@core/http';
import ws from '@core/ws';

import type { GroupSceneDetail, SceneDetail, SceneInfo } from './type';

const enum API {
  获取场景 = '/scene/getById',
  保存场景 = '/scene/saveById',
  推送场景 = '/scene/pushById',

  获取组场景 = '/scene/getByGroupId',
  保存组场景 = '/scene/saveByGroupId',

  实时监控场景 = '/scene/monitor/:id',
  实时监控真实场景 = '/scene/monitor/real/:id',
  实时监控库位状态 = '/ws/storage-location/:id',
}

export async function getSceneById(id: SceneInfo['id']): Promise<SceneDetail | null> {
  if (!id) return null;
  type B = { id: string };
  type D = SceneDetail;
  try {
    const body = { id };
    const data = await http.post<D, B>(API.获取场景, body);
    return data ?? null;
  } catch (error) {
    console.debug(error);
    return null;
  }
}

export async function saveSceneById(id: SceneInfo['id'], json: string, png?: string): Promise<boolean> {
  if (!id) return false;
  type B = { id: string; json: string; png?: string };
  type D = void;
  try {
    const body = { id, json, png };
    await http.post<D, B>(API.保存场景, body);
    return true;
  } catch (error) {
    console.debug(error);
    return false;
  }
}

export async function pushSceneById(id: SceneInfo['id']): Promise<boolean> {
  if (!id) return false;
  type B = { id: string };
  type D = void;
  try {
    const body = { id };
    await http.post<D, B>(API.推送场景, body);
    return true;
  } catch (error) {
    console.debug(error);
    return false;
  }
}

export async function getSceneByGroupId(
  id: RobotGroup['id'],
  sid: RobotGroup['sid'],
): Promise<GroupSceneDetail | null> {
  if (!id || !sid) return null;
  type B = { id: string; sid: string };
  type D = GroupSceneDetail;
  try {
    const body = { id, sid };
    const data = await http.post<D, B>(API.获取组场景, body);
    return data ?? null;
  } catch (error) {
    console.debug(error);
    return null;
  }
}

export async function saveSceneByGroupId(id: RobotGroup['id'], sid: RobotGroup['sid'], json: string): Promise<boolean> {
  if (!id || !sid) return false;
  type B = { id: string; sid: string; json: string };
  type D = void;
  try {
    const body = { id, sid, json };
    await http.post<D, B>(API.保存组场景, body);
    return true;
  } catch (error) {
    console.debug(error);
    return false;
  }
}

export async function monitorStorageStatusById(id: SceneInfo['id']): Promise<WebSocket | null> {
  if (!id) return null;
  try {
    const socket = await ws.create(API.实时监控库位状态.replace(':id', id));
    return socket;
  } catch (error) {
    console.debug(error);
    return null;
  }
}

export async function monitorSceneById(id: SceneInfo['id']): Promise<WebSocket | null> {
  if (!id) return null;
  try {
    const socket = await ws.create(API.实时监控场景.replace(':id', id));
    return socket;
  } catch (error) {
    console.debug(error);
    return null;
  }
}

export async function monitorRealSceneById(id: SceneInfo['id']): Promise<WebSocket | null> {
  if (!id) return null;
  try {
    const socket = await ws.create(API.实时监控真实场景.replace(':id', id));
    return socket;
  } catch (error) {
    console.debug(error);
    return null;
  }
}

export async function monitorStorageLocationById(
  id: SceneInfo['id'],
  options?: {
    interval?: number;
    storage_area_id?: string;
    station_name?: string;
    layer_name?: string;
    is_occupied?: boolean;
    is_locked?: boolean;
    is_disabled?: boolean;
  },
): Promise<WebSocket | null> {
  if (!id) return null;
  try {
    let url = API.实时监控库位状态.replace(':id', id);

    // 构建查询参数
    const params = new URLSearchParams();
    if (options?.interval !== undefined) params.append('interval', options.interval.toString());
    if (options?.storage_area_id) params.append('storage_area_id', options.storage_area_id);
    if (options?.station_name) params.append('station_name', options.station_name);
    if (options?.layer_name) params.append('layer_name', options.layer_name);
    if (options?.is_occupied !== undefined) params.append('is_occupied', options.is_occupied.toString());
    if (options?.is_locked !== undefined) params.append('is_locked', options.is_locked.toString());
    if (options?.is_disabled !== undefined) params.append('is_disabled', options.is_disabled.toString());

    if (params.toString()) {
      url += '?' + params.toString();
    }

    const socket = await ws.create(import.meta.env.ENV_STORAGE_WEBSOCKET_BASE + url);
    return socket;
  } catch (error) {
    console.debug(error);
    return null;
  }
}
